@@ -28,14 +28,14 @@
  ****************************************************************************/
 
 /****************************************************************************
- *   Author: Juergen Pfeifer <Juergen.Pfeifer@T-Online.de> 1997             *
+ *   Author: Juergen Pfeifer <juergen.pfeifer@gmx.net> 1997                 *
  ****************************************************************************/
 
 #include "cursesm.h"
 #include "cursesapp.h"
 #include "internal.h"
 
-MODULE_ID("$Id: cursesm.cc,v 1.8 1998/02/11 12:13:40 tom Exp $")
+MODULE_ID("$Id: cursesm.cc,v 1.11 1999/07/31 09:45:23 juergen Exp $")
   
 NCursesMenuItem::~NCursesMenuItem() {
   if (item)
@@ -306,7 +306,10 @@ NCursesMenu::operator()(void) {
 	  NCursesMenuItem* itm = current_item();
 	  assert(itm);
 	  if (itm->options() & O_SELECTABLE)
-	    b_action = itm->action();
+	    {
+	      b_action = itm->action();
+	      refresh();
+	    }
 	  else
 	    On_Not_Selectable(c);
 	}
@@ -316,6 +319,7 @@ NCursesMenu::operator()(void) {
 	    NCursesMenuItem* itm = my_items[i];
 	    if (itm->value()) {
 	      b_action |= itm->action();
+	      refresh();
 	    }
 	  }
 	}
