@@ -36,7 +36,7 @@
  */
 #include "panel.priv.h"
 
-MODULE_ID("$Id: p_move.c,v 1.2 1998/02/11 12:14:01 tom Exp $")
+MODULE_ID("$Id: p_move.c,v 1.3 1999/09/29 15:22:32 juergen Exp $")
 
 int
 move_panel(PANEL *pan, int starty, int startx)
@@ -46,14 +46,9 @@ move_panel(PANEL *pan, int starty, int startx)
   if(!pan)
     return(ERR);
   if(_nc_panel_is_linked(pan))
-    _nc_override(pan,P_TOUCH);
+    PANEL_UPDATE(pan,(PANEL*)0);
   win = pan->win;
   if(mvwin(win,starty,startx))
     return(ERR);
-  getbegyx(win, pan->wstarty, pan->wstartx);
-  pan->wendy = pan->wstarty + getmaxy(win);
-  pan->wendx = pan->wstartx + getmaxx(win);
-  if(_nc_panel_is_linked(pan))
-    _nc_calculate_obscure();
   return(OK);
 }
