@@ -5,15 +5,15 @@
  *
  * modified 10-18-89 for curses (jrl)
  * 10-18-89 added signal handling
+ *
+ * $Id: gdc.c,v 1.10 1997/10/18 20:06:06 tom Exp $
  */
 
+#include <test.priv.h>
+
 #include <time.h>
 #include <signal.h>
-#include <curses.h>
-#include <stdlib.h>
-#ifndef NONPOSIX
-#include <unistd.h>
-#endif
+#include <string.h>
 
 #define YBASE	10
 #define XBASE	10
@@ -21,31 +21,61 @@
 #define YDEPTH	5
 
 /* it won't be */
-long now; /* yeah! */
-struct tm *tm;
+static time_t now; /* yeah! */
+static struct tm *tm;
 
-short disp[11] = {
+static short disp[11] = {
 	075557, 011111, 071747, 071717, 055711,
 	074717, 074757, 071111, 075757, 075717, 002020
 };
-long old[6], next[6], new[6], mask;
-char scrol;
+static long older[6], next[6], newer[6], mask;
+static char scrol;
 
-int sigtermed=0;
+static int sigtermed = 0;
 
-int hascolor = 0;
+static int hascolor = 0;
 
-void set(int, int);
-void standt(int);
-void movto(int, int);
+static void set(int, int);
+static void standt(int);
+static void movto(int, int);
 
 static
-void sighndl(int signo)
+RETSIGTYPE sighndl(int signo)
 {
 	signal(signo, sighndl);
 	sigtermed=signo;
 }
 
+static void
+drawbox(void)
+{
+	chtype bottom[XLENGTH+1];
+	int n;
+
+	if(hascolor)
+		attrset(COLOR_PAIR(3));
+
+	mvaddch(YBASE - 1,  XBASE - 1, ACS_ULCORNER);
+	hline(ACS_HLINE, XLENGTH);
+	mvaddch(YBASE - 1,  XBASE + XLENGTH, ACS_URCORNER);
+
+	mvaddch(YBASE + YDEPTH,  XBASE - 1, ACS_LLCORNER);
+	mvinchnstr(YBASE + YDEPTH, XBASE, bottom, XLENGTH);
+	for (n = 0; n < XLENGTH; n++)
+		bottom[n] = ACS_HLINE | (bottom[n] & (A_ATTRIBUTES | A_COLOR));
+	mvaddchnstr(YBASE + YDEPTH, XBASE, bottom, XLENGTH);
+	mvaddch(YBASE + YDEPTH,  XBASE + XLENGTH, ACS_LRCORNER);
+
+	move(YBASE,  XBASE - 1);
+	vline(ACS_VLINE, YDEPTH);
+
+	move(YBASE,  XBASE + XLENGTH);
+	vline(ACS_VLINE, YDEPTH);
+
+	if(hascolor)
+		attrset(COLOR_PAIR(2));
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -61,17 +91,23 @@ int n = 0;
 	cbreak();
 	noecho();
 	nodelay(stdscr, 1);
-	
+    	curs_set(0);
+
 	hascolor = has_colors();
 
-	if(hascolor) {	
+	if(hascolor) {
+		int bg = COLOR_BLACK;
 		start_color();
+#ifdef NCURSES_VERSION
+		if (use_default_colors() == OK)
+			bg = -1;
+#endif
 		init_pair(1, COLOR_BLACK, COLOR_RED);
-		init_pair(2, COLOR_RED, COLOR_BLACK);
-		init_pair(3, COLOR_WHITE, COLOR_BLACK);
+		init_pair(2, COLOR_RED,   bg);
+		init_pair(3, COLOR_WHITE, bg);
 		attrset(COLOR_PAIR(2));
 	}
-	
+
 	clear();
 	refresh();
 	while(--argc > 0) {
@@ -81,25 +117,7 @@ int n = 0;
 			n = atoi(*argv);
 	}
 
-	if(hascolor) {	
-		attrset(COLOR_PAIR(3));
-
-		mvaddch(YBASE - 1,  XBASE - 1, ACS_ULCORNER);
-		hline(ACS_HLINE, XLENGTH);
-		mvaddch(YBASE - 1,  XBASE + XLENGTH, ACS_URCORNER);
-
-		mvaddch(YBASE + YDEPTH,  XBASE - 1, ACS_LLCORNER);
-		hline(ACS_HLINE, XLENGTH);
-		mvaddch(YBASE + YDEPTH,  XBASE + XLENGTH, ACS_LRCORNER);
-
-		move(YBASE,  XBASE - 1);
-		vline(ACS_VLINE, YDEPTH);
-
-		move(YBASE,  XBASE + XLENGTH);
-		vline(ACS_VLINE, YDEPTH);
-
-		attrset(COLOR_PAIR(2));
-	}
+	drawbox();
 	do {
 		char	buf[30];
 
@@ -117,15 +135,15 @@ int n = 0;
 		for(k=0; k<6; k++) {
 			if(scrol) {
 				for(i=0; i<5; i++)
-					new[i] = (new[i]&~mask) | (new[i+1]&mask);
-				new[5] = (new[5]&~mask) | (next[k]&mask);
+					newer[i] = (newer[i]&~mask) | (newer[i+1]&mask);
+				newer[5] = (newer[5]&~mask) | (next[k]&mask);
 			} else
-				new[k] = (new[k]&~mask) | (next[k]&mask);
+				newer[k] = (newer[k]&~mask) | (next[k]&mask);
 			next[k] = 0;
 			for(s=1; s>=0; s--) {
 				standt(s);
 				for(i=0; i<6; i++) {
-					if((a = (new[i]^old[i])&(s ? new : old)[i]) != 0) {
+					if((a = (newer[i]^older[i])&(s ? newer : older)[i]) != 0) {
 						for(j=0,t=1<<26; t; t>>=1,j++) {
 							if(a&t) {
 								if(!(a&(t<<1))) {
@@ -136,11 +154,15 @@ int n = 0;
 						}
 					}
 					if(!s) {
-						old[i] = new[i];
+						older[i] = newer[i];
 					}
 				}
 				if(!s) {
+					if (scrol)
+						drawbox();
 					refresh();
+					if (scrol)
+						napms(150);
 				}
 			}
 		}
@@ -151,6 +173,7 @@ int n = 0;
 		mvaddstr(16, 30, buf);
 
 		movto(6, 0);
+		drawbox();
 		refresh();
 		sleep(1);
 		while(wgetch(stdscr) != ERR)
@@ -159,19 +182,21 @@ int n = 0;
 			standend();
 			clear();
 			refresh();
+    			curs_set(1);
 			endwin();
 			fprintf(stderr, "gdc terminated by signal %d\n", sigtermed);
-			exit(1);
+			return EXIT_FAILURE;
 		}
 	} while(--n);
 	standend();
 	clear();
 	refresh();
+    	curs_set(1);
 	endwin();
-	return(0);
+	return EXIT_SUCCESS;
 }
 
-void
+static void
 set(int t, int n)
 {
 int i, m;
@@ -179,21 +204,21 @@ int i, m;
 	m = 7<<n;
 	for(i=0; i<5; i++) {
 		next[i] |= ((disp[t]>>(4-i)*3)&07)<<n;
-		mask |= (next[i]^old[i])&m;
+		mask |= (next[i]^older[i])&m;
 	}
 	if(mask&m)
 		mask |= m;
 }
 
-void
+static void
 standt(int on)
 {
 	if (on) {
 		if(hascolor) {
 			attron(COLOR_PAIR(1));
 		} else {
-			attron(A_STANDOUT);	
-		}	
+			attron(A_STANDOUT);
+		}
 	} else {
 		if(hascolor) {
 			attron(COLOR_PAIR(2));
@@ -203,9 +228,8 @@ standt(int on)
 	}
 }
 
-void
+static void
 movto(int line, int col)
 {
 	move(line, col);
 }
-
